#!/bin/sh

/root/mountpoint-genprog/genprog-many-bugs/php-bug-2011-10-31-2e5d5e5ac6-b5f15ef561/php/sapi/cgi/php-cgi -C   Array -f "/root/mountpoint-genprog/genprog-many-bugs/php-bug-2011-10-31-2e5d5e5ac6-b5f15ef561/php/ext/phar/tests/frontcontroller32.php"  2>&1
