<?php
echo "ok\n";
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comment comment comment comment comment comment comment
// comment comment comm
?>
